@@ -1,18 +1,18 @@
-/*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003-2017 Antonio Diaz Diaz.
+/* GNU Ocrad - Optical Character Recognition program
+   Copyright (C) 2003-2022 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -31,9 +31,9 @@
 #include "feats.h"
 
 
-// Looks for three black sections in column hcenter()  n, then tests if
-// upper and lower gaps are open to the right or to the left
-//
+/* Look for three black sections in column hcenter()  n, then test whether
+   upper and lower gaps are open to the right or to the left.
+*/
 int Features::test_235Esz( const Charset & charset ) const
   {
   const int csize = 3;
@@ -506,7 +506,7 @@ int Features::test_G() const
   }
 
 
-// Common feature: U-shaped top of character
+// common feature: U-shaped top of character
 //
 int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const
   {
@@ -634,9 +634,9 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const
   }
 
 
-// Looks for the nearest frontier in column hcenter(), then tests if
-// gap is open downwards (except for 'x')
-//
+/* Look for the nearest frontier in column hcenter(), then test whether gap
+   is open downwards (except for 'x').
+*/
 int Features::test_hknwx( const Rectangle & charbox ) const
   {
   const int m8 = tp.minima( b.height() / 8 );
@@ -684,9 +684,9 @@ int Features::test_hknwx( const Rectangle & charbox ) const
   }
 
 
-// Looks for four black sections in column hcenter()  1, then tests if
-// upper gap is open to the right and lower gaps are open to the left
-//
+/* Look for four black sections in column hcenter()  1, then test whether
+   upper gap is open to the right and lower gaps are open to the left.
+*/
 int Features::test_s_cedilla() const
   {
   int urow2 = 0, urow3 = 0, urow4 = 0, col, black_section = 0;
@@ -757,9 +757,9 @@ int Features::test_easy( const Rectangle & charbox ) const
   }
 
 
-// Recognizes single line, non-rectangular characters without holes.
-// '/<>C[\^`c
-//
+/* Recognize single line, non-rectangular characters without holes.
+   '/<>C[\^`c
+*/
 int Features::test_line( const Rectangle & charbox ) const
   {
   const int vnoise = ( b.height() / 30 ) + 1;
